==> XML-XPath-0.50.tar.gz: XML-XPath-0.50/XPath/Builder.pm <==
# $Id: Builder.pm,v 1.3 2000/05/08 13:08:01 matt Exp $

package XML::XPath::Builder;

use strict;

# to get array index constants
use XML::XPath::Node;
use XML::XPath::Node::Element;
use XML::XPath::Node::Attribute;
use XML::XPath::Node::Namespace;
use XML::XPath::Node::Text;
use XML::XPath::Node::PI;
use XML::XPath::Node::Comment;

sub new {
	my $class = shift;
	my $self = ($#_ == 0) ? { %{ (shift) } } : { @_ };

	bless $self, $class;
}


# SHOWSTOPPER!
#
# Perl SAX doesn't yet define how to handle namespaces

sub mkelement {
	my ($self, $tag, $attribs) = @_;

	my $node = XML::XPath::Node::Element->new($tag, '#default');
	
	while (@$attribs) {
		my ($key, $val) = (shift @$attribs, shift @$attribs);
		my $newattr = XML::XPath::Node::Attribute->new($key, $val);
		$node->appendAttribute($newattr);
	}
	
	return $node;
}

sub start_document {
	my $self = shift;
	$self->{Current} = XML::XPath::Node::Element->new();
	$self->{Root} = $self->{Current};
}

sub characters {
	my $self = shift;
	my $characters = shift;
	
	my @kids = $self->{Current}->getChildNodes;
	if (@kids && $kids[-1]->getNodeType == TEXT_NODE) {
		$kids[-1]->appendText($characters);
	}
	else {
		my $node = XML::XPath::Node::Text->new($characters);
		$self->{Current}->appendChild($node);
	}
}

sub start_element {
	my $self = shift;
	my $element = shift;
	my $node = mkelement($self, $element->{Name}, $element->{Attributes});
	$self->{Current}->appendChild($node);
	$self->{Current} = $node;
}

sub end_element {
	my $self = shift;
	$self->{Last} = $self->{Current};
	$self->{Current} = $self->{Current}->getParentNode;
}

sub end_document {
	my $self = shift;
	
	delete $self->{Last};
	delete $self->{Current};
	return $self->{Root};
}

sub processing_instruction {
	my $self = shift;
	my $pi = shift;
	my $node = XML::XPath::Node::PI->new($pi->{Target}, $pi->{Data});
	$self->{Current}->appendChild($node);
}

sub comment {
	my $self = shift;
	my $comment = shift;
	my $node = XML::XPath::Node::Comment->new($comment);
	$self->{Current}->appendChild($node);
}

1;

__END__

=head1 NAME

XML::XPath::Builder - SAX handler for building an XPath tree

=head1 SYNOPSIS

 use AnySAXParser;
 use XML::XPath::Builder;

 $builder = XML::XPath::Builder->new();
 $parser = AnySAXParser->new( Handler => $builder );

 $root_node = $parser->parse( Source => [SOURCE] );

=head1 DESCRIPTION

C<XML::XPath::Builder> is a SAX handler for building an XML::XPath
tree.

C<XML::XPath::Builder> is used by creating a new instance of
C<XML::XPath::Builder> and providing it as the Handler for a SAX
parser.  Calling `C<parse()>' on the SAX parser will return the
root node of the tree built from that parse.

=head1 AUTHOR

Ken MacLeod, <ken@bitsko.slc.ut.us>

=head1 SEE ALSO

perl(1), XML::XPath(3)

PerlSAX.pod in libxml-perl

Extensible Markup Language (XML) <http://www.w3c.org/XML>

=cut

==> XML-XPath-0.52.tar.gz: XML-XPath-0.52/XPath/Builder.pm <==
# $Id: Builder.pm,v 1.4 2000/05/28 09:50:53 matt Exp $

package XML::XPath::Builder;

use strict;

# to get array index constants
use XML::XPath::Node;
use XML::XPath::Node::Element;
use XML::XPath::Node::Attribute;
use XML::XPath::Node::Namespace;
use XML::XPath::Node::Text;
use XML::XPath::Node::PI;
use XML::XPath::Node::Comment;

sub new {
	my $class = shift;
	my $self = ($#_ == 0) ? { %{ (shift) } } : { @_ };

	bless $self, $class;
}


# SHOWSTOPPER!
#
# Perl SAX doesn't yet define how to handle namespaces

sub mkelement {
	my ($self, $tag, $attribs) = @_;

	my $node = XML::XPath::Node::Element->new($tag, '#default');
	
	for my $attr (keys %$attribs) {
		my $newattr = XML::XPath::Node::Attribute->new($attr, $attribs->{$attr});
		$node->appendAttribute($newattr);
	}
	
	return $node;
}

sub start_document {
	my $self = shift;
	$self->{Current} = XML::XPath::Node::Element->new();
	$self->{Root} = $self->{Current};
}

sub characters {
	my $self = shift;
	my $characters = shift;
	
	my @kids = $self->{Current}->getChildNodes;
	if (@kids && $kids[-1]->getNodeType == TEXT_NODE) {
		$kids[-1]->appendText($characters->{Data});
	}
	else {
		my $node = XML::XPath::Node::Text->new($characters->{Data});
		$self->{Current}->appendChild($node);
	}
}

sub start_element {
	my $self = shift;
	my $element = shift;
	my $node = mkelement($self, $element->{Name}, $element->{Attributes});
	$self->{Current}->appendChild($node);
	$self->{Current} = $node;
}

sub end_element {
	my $self = shift;
	$self->{Last} = $self->{Current};
	$self->{Current} = $self->{Current}->getParentNode;
}

sub end_document {
	my $self = shift;
	
	delete $self->{Last};
	delete $self->{Current};
	return $self->{Root};
}

sub processing_instruction {
	my $self = shift;
	my $pi = shift;
	my $node = XML::XPath::Node::PI->new($pi->{Target}, $pi->{Data});
	$self->{Current}->appendChild($node);
}

sub comment {
	my $self = shift;
	my $comment = shift;
	my $node = XML::XPath::Node::Comment->new($comment->{Data});
	$self->{Current}->appendChild($node);
}

1;

__END__

=head1 NAME

XML::XPath::Builder - SAX handler for building an XPath tree

=head1 SYNOPSIS

 use AnySAXParser;
 use XML::XPath::Builder;

 $builder = XML::XPath::Builder->new();
 $parser = AnySAXParser->new( Handler => $builder );

 $root_node = $parser->parse( Source => [SOURCE] );

=head1 DESCRIPTION

C<XML::XPath::Builder> is a SAX handler for building an XML::XPath
tree.

C<XML::XPath::Builder> is used by creating a new instance of
C<XML::XPath::Builder> and providing it as the Handler for a SAX
parser.  Calling `C<parse()>' on the SAX parser will return the
root node of the tree built from that parse.

=head1 AUTHOR

Ken MacLeod, <ken@bitsko.slc.ut.us>

=head1 SEE ALSO

perl(1), XML::XPath(3)

PerlSAX.pod in libxml-perl

Extensible Markup Language (XML) <http://www.w3c.org/XML>

=cut
